//************************************************
// Test_Romeo.c
//************************************************
// Tutorial XFT35-05: Hardware implementation of
// fuzzy controllers with xfsg
//************************************************

#include <stdio.h>              /* Standard input/output definitions */
#include "Test_Romeo.h"
#include "xparameters.h"
#include "xil_cache.h"
#include "backward_ip.h"

// Define delay length
#define DELAY 10
#define BACKWARD_IP_DEVICE_ID                  XPAR_BACKWARD_IP_0_DEVICE_ID

backward_ip backward_ip_Ps;

int main (int argc, char *argv[]) {

   int x, y, phi;
   int bw;
   double sref, gref;
   int i, Success;
   int count = 0;

// initialize the software driver
   Success = backward_ip_Initialize(&backward_ip_Ps, BACKWARD_IP_DEVICE_ID);
			if (Success != XST_SUCCESS) {
				xil_printf("Initialization Failure\n\r");
				return XST_FAILURE;
			}

// ---------
// initialize Romeo's Status
// ---------

  status.x = 0.0;          //   0.0    6.0  -10.0
  status.y = 7.0;          //   7.0    6.0    3.0
  status.angle = 170.0;    // 170.0  -45.0    0.0
  status.speed = -1.0;
  status.gamma = 0.0;
   
/*  printf ("\n\r Posicion inicial: x= %6.2f, y= %6.2f, angle= %6.2f, speed= %6.2f, gamma= %6.2f\n\n\r",
          status.x, status.y, status.angle, status.speed, status.gamma);  */

  do {
 
	// Calculate input values
	x = (status.x+20)*1023/40;
	phi = (status.angle+180)*1023/360;
	y = (status.y+2)*1023/16;

	// Write input values to AXI registers
	backward_ip_x_write(&backward_ip_Ps, x);
	backward_ip_y_write(&backward_ip_Ps, y);
	backward_ip_phi_write(&backward_ip_Ps, phi);

	// Wait for completing the inference process
	for (i=0; i<DELAY; i++);

	// Read output data from AXI registers
	bw = backward_ip_bw_read(&backward_ip_Ps);
	gref = ((bw/1023.)*0.8)-0.4;
	sref = status.speed;
	
	// Actualize Romeo's status
	Romeo(sref, gref);

	count++;
 	printf(" %5.3f %5.3f %5.3f %5.3f %5.3f \n\r",
 			status.x, status.y, status.angle, status.speed, status.gamma);
  }
  while( status.y >= 0 && status.y <= 20 && count < 250);
  
  Xil_ICacheDisable();
  Xil_DCacheDisable();

  return 0; 
}

